/* *************************************************************************
 * Copyright (C) 2017 - Trelleborg Marine Systems
 * Author		:	Vincent Vuarnoz
 * Date			:	04 Sep 2017
 * Description	:	See Header.
 * ************************************************************************/

/* Includes */
#include "debug.h"

// Standard includes
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

extern int vsnprintf (char * s, size_t n, const char * format, va_list arg );


static UART_Handle mHandle = NULL;


/* Public methods declarations */

void debug_init(void) {
	UART_Params uartParams;

	UART_init();
	UART_Params_init(&uartParams);

	uartParams.writeDataMode    = UART_DATA_BINARY;
	uartParams.readDataMode     = UART_DATA_BINARY;
	uartParams.readReturnMode   = UART_RETURN_FULL;
	uartParams.readEcho         = UART_ECHO_OFF;
	uartParams.baudRate         = 115200;

	mHandle = UART_open(Board_UART0, &uartParams);
	if(mHandle == NULL)
		while(1); // ERror !
}





//*****************************************************************************
//
//! prints the formatted string on to the console
//!
//! \param[in]  format  - is a pointer to the character string specifying the
//!                       format in the following arguments need to be
//!                       interpreted.
//! \param[in]  [variable number of] arguments according to the format in the
//!             first parameters
//!
//! \return count of characters printed
//
//*****************************************************************************
int Report(const uint8_t *pcFormat, ...)
{
    int     iRet = 0;
    char        *pcBuff;
    char        *pcTemp;
    int         iSize = 256;
    va_list     list;


    pcBuff = (char*)malloc(iSize);
    if(pcBuff == NULL)
    {
        return -1;
    }
    while(1)
    {
        va_start(list,pcFormat);
        iRet = vsnprintf(pcBuff, iSize, (const char*)pcFormat, list);
        va_end(list);
        if((iRet > -1) && (iRet < iSize))
        {
            break;
        }
        else
        {
            iSize *= 2;
            if((pcTemp = realloc(pcBuff, iSize)) == NULL)
            {
                Message("Could not reallocate memory\n\r");
                iRet = -1;
                break;
            }
            else
            {
                pcBuff = pcTemp;
            }
        }
    }
    Message((const uint8_t*)pcBuff);
    free(pcBuff);

    return iRet;
}


//*****************************************************************************
//
//! Outputs a character string to the console
//!
//! This function
//!        1. prints the input string character by character on to the console.
//!
//! \param[in]  str - is the pointer to the string to be printed
//!
//! \return none
//!
//! \note If UART_NONPOLLING defined in than Message or UART write should be
//!       called in task/thread context only.
//
//*****************************************************************************
void Message(const uint8_t *str)
{
	UART_writePolling(mHandle, str, strlen((const char*)str));
}

