/*
 * Copyright (c) 2015-2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== sl_Start_hangs_demo.c ========
 */

#include "timer.h"
#include "watchdog.h"
#include "debug.h"
#include "wifi_interface.h"

#include <stdint.h>
#include <stddef.h>

/* Driver Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/net/wifi/device.h>

/* Example/Board Header files */
#include "Board.h"


static void sDisplayAppBanner(void);
static void very_simple_process(void);


// Number of iterations of the "very_simple_process()" in between calls to the wifi_process
#define NUM_ITER 20

/*
 *  ======== mainThread ========
 */
void *mainThread(void *arg0)
{
	uint32_t ii;

	/* Call driver init functions */
	GPIO_init();

	/* Modules init */
	timer_init();
	watchdog_init();
	debug_init();
	sDisplayAppBanner();


	wifi_interface_init();


	while(1) {
		/* The SimpleLink host driver architecture mandate calling 'sl_task' in a NO-RTOS application's main loop.       */
		/* The purpose of this call, is to handle asynchronous events and get flow control information sent from the NWP.*/
		/* Every event is classified and later handled by the host driver event handlers.                                */
		sl_Task(NULL);

		wifi_interface_process();

		for(ii = 0; ii < NUM_ITER; ++ii) {
			sl_Task(NULL);
			very_simple_process();
		}
	}
}




static void sDisplayAppBanner(void) {

     DEBUG_INFO("\r\n");
     DEBUG_INFO("\t");
     DEBUG_INFO("============================================");
     DEBUG_INFO("\r\n");
     DEBUG_INFO("\t   sl_Start hangs bug demo\r\n");
     DEBUG_INFO("\t");
     DEBUG_INFO("============================================");
     DEBUG_INFO("\r\n");
     DEBUG_INFO("\r\n");
}

static void very_simple_process(void) {
	static bool toggle = false;

	if(toggle) {
		toggle = 0;
	}
	else {
		toggle = 1;
	}
}
