/* *************************************************************************
 * Copyright (C) 2017 - Trelleborg Marine Systems
 * Author		:	Vincent Vuarnoz
 * Date			:	13 Sep 2017
 * Description	:	See header file
 * *************************************************************************
 */

#include "timer.h"

/* TI-specific Header files */
#include <Board.h>
#include <ti/drivers/Timer.h>
#include <stddef.h>


/* Module Variables */
static Timer_Handle mHandleTimer0;
static uint32_t mMilliSeconds = 0;
static uint32_t m10MicroSeconds = 0;
static uint32_t mUsHelperCount = 0;

/* Callback Functions Declarations */
void timer0Callback(Timer_Handle handle);


/* Functions Definitions */
void timer_init(void) {

	Timer_Params params;

	Timer_init();

	// Timer 0
	Timer_Params_init(&params);
	params.periodUnits = Timer_PERIOD_US;
	params.period = 10;
	params.timerMode = Timer_CONTINUOUS_CALLBACK;
	params.timerCallback = timer0Callback;

	mHandleTimer0 = Timer_open(Board_TIMER0, &params);
	if (mHandleTimer0 == NULL) {
#ifdef _DEBUG_
		while (1);
#endif
	}

	int32_t status = Timer_start(mHandleTimer0);
	if (status == Timer_STATUS_ERROR) {
#ifdef _DEBUG_
		while (1);
#endif
	}
}

uint32_t timer_get_count_ms(void) {
	return mMilliSeconds;
}

uint32_t timer_get_count_10us(void) {
	return m10MicroSeconds;
}

void timer0Callback(Timer_Handle handle) {

	++m10MicroSeconds;
	++mUsHelperCount;

	if(mUsHelperCount >= 100) {
		++mMilliSeconds;
		mUsHelperCount = 0;
	}
}
