/* *************************************************************************
 * Copyright (C) 2017 - Trelleborg Marine Systems
 * Author		:	Vincent Vuarnoz
 * Date			:	04 Sep 2017
 * Description	:	See header file
 * *************************************************************************
 */


/* Includes */
#include "wifi_interface.h"
#include "clock.h"
#include "debug.h"

/* TI-DRIVERS Header files */
#include <ti/drivers/net/wifi/device.h>
#include <ti/drivers/net/wifi/simplelink.h>
#include <ti/drivers/net/wifi/netcfg.h>
#include <ti/drivers/SPI.h>

/* Defines */



/* Struct / Typedef */

/* Private Variables */
static uint32_t mState;

#define wifi_interface_debug(x, ...) DEBUG_INFO(x, ##__VA_ARGS__)
#define wifi_interface_debug_extra(x, ...) //DEBUG_INFO(x, ##__VA_ARGS__)

#define wifi_driver_debug(x, ...)
#define wifi_driver_error(x, ...) DEBUG_ERROR(x, ##__VA_ARGS__)

/* Static Functions */


/* Definitions */
void wifi_interface_init(void) {

	SPI_init();

	mState = 0;
}



void wifi_interface_process(void) {

	static uint32_t mGeneralTimer;
	_i16 Status;

	switch(mState) {
	case 0:
		wifi_interface_debug("Wifi init...\r\n");
		++mState;
		break;
	case 1:
		wifi_interface_debug_extra("sl_Start\r\n");
		Status = sl_Start(0, 0, 0);
		mState = 20;
		break;
	case 20:
		wifi_interface_debug_extra("set ROLA STA\r\n");
		sl_WlanSetMode(ROLE_STA);
		++mState;
		break;
	case 21:
		wifi_interface_debug_extra("sl_Stop\r\n");
		Status = sl_Stop(0);
		wifi_interface_debug_extra("sl_Start\r\n");
		Status = sl_Start(0, 0, 0);
		mState = 23;
		break;
	case 23: {
		wifi_interface_debug_extra("Scanning...\r\n");
		SlWlanNetworkEntry_t netEntries[30];
		_i16 resultsCount = sl_WlanGetNetworkList(0, 30, &netEntries[0]);
		if(resultsCount == SL_ERROR_WLAN_GET_NETWORK_LIST_EAGAIN) {
			mGeneralTimer = clock_get_current_time();
			mState = 24;
		}
		else if(resultsCount >= 0) {
			mState = 30;
		}
		break;
	}
	case 24:
		if(clock_diff_to_now(mGeneralTimer) > 2000) {
			mState = 23;
		}
		break;
	case 30:
		wifi_interface_debug_extra("set ROLA AP\r\n");
		sl_WlanSetMode(ROLE_AP);
		++mState;
		break;
	case 31:
		wifi_interface_debug_extra("sl_Stop II\r\n");
		Status = sl_Stop(0);
		wifi_interface_debug_extra("sl_Start II\r\n");
		Status = sl_Start(0, 0, 0);
		mState = 33;
		break;
	case 33:
		if(Status < 0) {
			wifi_interface_debug("Failed! (error code: %i)\r\n", Status);
		}
		else {
			wifi_interface_debug("Success! :)\r\n");
		}
		mState = 300;
		break;
	case 300:
	default:
		break;
	}

}




















/* *************************************************************************
 *                        Callback Functions
 * ************************************************************************/



/*!
    \brief          SimpleLinkWlanEventHandler

    This handler gets called whenever a WLAN event is reported
    by the host driver / NWP. Here user can implement he's own logic
    for any of these events. This handler is used by 'network_terminal'
    application to show case the following scenarios:

    1. Handling connection / Disconnection.
    2. Handling Addition of station / removal.
    3. RX filter match handler.
    4. P2P connection establishment.

    \param          pWlanEvent       -   pointer to Wlan event data.

    \return         void

    \note           For more information, please refer to: user.h in the porting
                    folder of the host driver and the  CC3120/CC3220 NWP programmer's
                    guide (SWRU455) sections 4.3.4, 4.4.5 and 4.5.5.

    \sa             cmdWlanConnectCallback, cmdEnableFilterCallback, cmdWlanDisconnectCallback,
                    cmdP2PModecallback.

*/
void SimpleLinkWlanEventHandler(SlWlanEvent_t *pWlanEvent)
{
    if(!pWlanEvent)
    {
        return;
    }

    switch(pWlanEvent->Id)
    {
        case SL_WLAN_EVENT_CONNECT:
        {
            wifi_driver_debug("\n\r[WLAN EVENT] STA Connected to the AP\n\r");
        }
        break;

        case SL_WLAN_EVENT_DISCONNECT:
        {
            SlWlanEventDisconnect_t  *pEventData = NULL;

            pEventData = &pWlanEvent->Data.Disconnect;

            /* If the user has initiated 'Disconnect' request,
              'reason_code' is SL_WLAN_DISCONNECT_USER_INITIATED */
            if(SL_WLAN_DISCONNECT_USER_INITIATED == pEventData->ReasonCode)
            {
                wifi_driver_debug("\n\r[WLAN EVENT] Device disconnected from the AP\n\r");
            }
            else
            {
                wifi_driver_debug("\n\r[WLAN ERROR] Device disconnected from the AP\n\r");
            }
        }
        break;

        case SL_WLAN_EVENT_PROVISIONING_STATUS:
        {
            /* Do nothing, this suppress provisioning event is because simplelink is configured to default state. */
        }
        break;

        case SL_WLAN_EVENT_STA_ADDED:
        {
            wifi_driver_debug("\n\r[WLAN EVENT] STA was added to AP\n\r");
        }
        break;

        case SL_WLAN_EVENT_STA_REMOVED:
        {
            wifi_driver_debug("\n\r[WLAN EVENT] STA was removed from AP\n\r");
        }
        break;

        case SL_WLAN_EVENT_RXFILTER:
        {
            SlWlanEventRxFilterInfo_t  *triggred_filter = NULL;

            triggred_filter = &(pWlanEvent->Data.RxFilterInfo) ;

            wifi_driver_debug("\n\r[WLAN EVENT] Rx filter match triggered. Set filters in filter bitmap :0x%x.\n\r", triggred_filter->UserActionIdBitmap);

            /*
             *     User can write he's / her's rx filter match handler here.
             *     Be advised, you can use the 'triggred_filter' structure info to determine which filter
             *     has received a match. (Bit X is set if user action id X was passed to a filter that matched a packet.)
             */
        }
        break;

        case SL_WLAN_EVENT_P2P_DEVFOUND:
        {
            wifi_driver_debug("\n\r[WLAN EVENT] P2P Remote device found\n\r");
//            sem_post(&(app_CB.P2P_CB.DeviceFound));
        }
        break;

        case SL_WLAN_EVENT_P2P_REQUEST:
        {
            wifi_driver_debug("\n\r[WLAN EVENT] P2P Negotiation request received\n\r");

//            /* This information is needed to create connection*/
//            memset(&(app_CB.P2P_CB.p2pPeerDeviceName), '\0', sizeof(app_CB.P2P_CB.p2pPeerDeviceName));
//            memcpy(&app_CB.P2P_CB.p2pPeerDeviceName,
//                    pWlanEvent->Data.P2PRequest.GoDeviceName,
//                    pWlanEvent->Data.P2PRequest.GoDeviceNameLen);

//            sem_post(&app_CB.P2P_CB.RcvNegReq);
        }
        break;

        case SL_WLAN_EVENT_P2P_CONNECT:
        {
            wifi_driver_debug("n\r[WLAN EVENT] P2P connection was successfully completed as CLIENT\n\r");
            wifi_driver_debug("n\rBSSID is %02x:%02x:%02x:%02x:%02x:%02x\n\r",
                                            pWlanEvent->Data.STAAdded.Mac[0],
                                            pWlanEvent->Data.STAAdded.Mac[1],
                                            pWlanEvent->Data.STAAdded.Mac[2],
                                            pWlanEvent->Data.STAAdded.Mac[3],
                                            pWlanEvent->Data.STAAdded.Mac[4],
                                            pWlanEvent->Data.STAAdded.Mac[5]);

//            sem_post(&app_CB.P2P_CB.RcvConReq);
        }
        break;

        case SL_WLAN_EVENT_P2P_CLIENT_ADDED:
        {
            wifi_driver_debug("n\r[WLAN EVENT] P2P connection was successfully completed as GO\n\r");
            wifi_driver_debug("n\rBSSID is %02x:%02x:%02x:%02x:%02x:%02x\n\r",
                                            pWlanEvent->Data.P2PClientAdded.Mac[0],
                                            pWlanEvent->Data.P2PClientAdded.Mac[1],
                                            pWlanEvent->Data.P2PClientAdded.Mac[2],
                                            pWlanEvent->Data.P2PClientAdded.Mac[3],
                                            pWlanEvent->Data.P2PClientAdded.Mac[4],
                                            pWlanEvent->Data.P2PClientAdded.Mac[5]);

//            sem_post(&app_CB.P2P_CB.RcvConReq);
        }
        break;

        case SL_WLAN_EVENT_P2P_DISCONNECT:
        {
            wifi_driver_debug("\n\r[WLAN EVENT] STA disconnected from device.\n\r");
//            CLR_STATUS_BIT(app_CB.Status ,STATUS_BIT_CONNECTION);
        }
        break;

        default:
        {
        	wifi_driver_error("\n\r[WLAN EVENT] Unexpected event [0x%x]\n\r", pWlanEvent->Id);
        }
        break;
    }
//    UART_PRINT(cmdPromptStr);
}

/*!
    \brief          SimpleLinkNetAppEventHandler

    This handler gets called whenever a Netapp event is reported
    by the host driver / NWP. Here user can implement he's own logic
    for any of these events. This handler is used by 'network_terminal'
    application to show case the following scenarios:

    1. Handling IPv4 / IPv6 IP address acquisition.
    2. Handling IPv4 / IPv6 IP address Dropping.

    \param          pNetAppEvent     -   pointer to Netapp event data.

    \return         void

    \note           For more information, please refer to: user.h in the porting
                    folder of the host driver and the  CC3120/CC3220 NWP programmer's
                    guide (SWRU455) section 5.7

*/
void SimpleLinkNetAppEventHandler(SlNetAppEvent_t *pNetAppEvent)
{
    if(!pNetAppEvent) {
        return;
    }

    switch(pNetAppEvent->Id)
    {
        case SL_DEVICE_EVENT_DROPPED_NETAPP_IPACQUIRED:
        {
            SlIpV4AcquiredAsync_t *pEventData = NULL;

            /* Ip Acquired Event Data */
            pEventData = &pNetAppEvent->Data.IpAcquiredV4;

            wifi_driver_debug("\n\r[NETAPP EVENT] IP set to: IPv4=%d.%d.%d.%d\n\r",

                    SL_IPV4_BYTE(pEventData->Ip,3),
                    SL_IPV4_BYTE(pEventData->Ip,2),
                    SL_IPV4_BYTE(pEventData->Ip,1),
                    SL_IPV4_BYTE(pEventData->Ip,0));
        }
        break;

        case SL_DEVICE_EVENT_DROPPED_NETAPP_IPACQUIRED_V6:
        {
            wifi_driver_debug("\n\r[NETAPP EVENT] IP Acquired: IPv6=");
        }
        break;

        case SL_DEVICE_EVENT_DROPPED_NETAPP_IP_LEASED:
        {
            wifi_driver_debug("\n\r[NETAPP EVENT] IP Leased to Client: IP=%d.%d.%d.%d \n\r",
                        SL_IPV4_BYTE(pNetAppEvent->Data.IpLeased.IpAddress, 3),
						SL_IPV4_BYTE(pNetAppEvent->Data.IpLeased.IpAddress, 2),
                        SL_IPV4_BYTE(pNetAppEvent->Data.IpLeased.IpAddress, 1),
						SL_IPV4_BYTE(pNetAppEvent->Data.IpLeased.IpAddress, 0));
        }
        break;

        case SL_DEVICE_EVENT_DROPPED_NETAPP_IP_RELEASED:
        {
            wifi_driver_debug("\n\r[NETAPP EVENT] IP is released.\n\r");
        }
        break;

        default:
        {
        	wifi_driver_error("\n\r[NETAPP EVENT] Unexpected event [0x%x] \n\r", pNetAppEvent->Id);
        }
        break;
    }
//    UART_PRINT(cmdPromptStr);
}

/*!
    \brief          SimpleLinkHttpServerEventHandler

    This handler gets called whenever a HTTP event is reported
    by the NWP internal HTTP server.

    \param          pHttpEvent       -   pointer to http event data.

    \param          pHttpEvent       -   pointer to http response.

    \return         void

    \note           For more information, please refer to: user.h in the porting
                    folder of the host driver and the  CC3120/CC3220 NWP programmer's
                    guide (SWRU455) chapter 9.

*/
void SimpleLinkHttpServerEventHandler(SlNetAppHttpServerEvent_t *pHttpEvent,
                                      SlNetAppHttpServerResponse_t *pHttpResponse)
{
    /* Unused in this application */
	wifi_driver_debug("\n\r[HTTP SERVER EVENT]  Ev = %u, Resp = %u\r\n", pHttpEvent->Event, pHttpResponse->Response);
}

/*!
    \brief          SimpleLinkGeneralEventHandler

    This handler gets called whenever a general error is reported
    by the NWP / Host driver. Since these errors are not fatal,
    application can handle them.

    \param          pDevEvent    -   pointer to device error event.

    \return         void

    \note           For more information, please refer to: user.h in the porting
                    folder of the host driver and the  CC3120/CC3220 NWP programmer's
                    guide (SWRU455) section 17.9.

*/
void SimpleLinkGeneralEventHandler(SlDeviceEvent_t *pDevEvent)
{
    if(!pDevEvent)
    {
        return;
    }
    /*
      Most of the general errors are not FATAL are are to be handled
      appropriately by the application
    */
    wifi_driver_debug("\n\r[GENERAL EVENT] - ID=[%d] Sender=[%d]\n\n",
               pDevEvent->Data.Error.Code,
               pDevEvent->Data.Error.Source);
}

/*!
    \brief          SimpleLinkSockEventHandler

    This handler gets called whenever a socket event is reported
    by the NWP / Host driver.

    \param          SlSockEvent_t    -   pointer to socket event data.

    \return         void

    \note           For more information, please refer to: user.h in the porting
                    folder of the host driver and the  CC3120/CC3220 NWP programmer's
                    guide (SWRU455) section 7.6.

*/
void SimpleLinkSockEventHandler(SlSockEvent_t *pSock)
{
    /* Unused in this application */
//	UART_PRINT("\n\r[SOCK EVENT]  Ev = %u\r\n", pSock->Event);
}

/*!
    \brief          SimpleLinkFatalErrorEventHandler

    This handler gets called whenever a socket event is reported
    by the NWP / Host driver. After this routine is called, the user's
    application must restart the device in order to recover.

    \param          slFatalErrorEvent    -   pointer to fatal error event.

    \return         void

    \note           For more information, please refer to: user.h in the porting
                    folder of the host driver and the  CC3120/CC3220 NWP programmer's
                    guide (SWRU455) section 17.9.

*/
void SimpleLinkFatalErrorEventHandler(SlDeviceFatal_t *slFatalErrorEvent)
{
    switch (slFatalErrorEvent->Id)
    {
        case SL_DEVICE_EVENT_FATAL_DEVICE_ABORT:
        {
        	wifi_driver_error("\n\r[ERROR] - FATAL ERROR: Abort NWP event detected: "
                        "AbortType=%d, AbortData=0x%x\n\r",
                        slFatalErrorEvent->Data.DeviceAssert.Code,
                        slFatalErrorEvent->Data.DeviceAssert.Value);
        }
        break;

        case SL_DEVICE_EVENT_FATAL_DRIVER_ABORT:
        {
        	wifi_driver_error("\n\r[ERROR] - FATAL ERROR: Driver Abort detected. \n\r");
        }
        break;

        case SL_DEVICE_EVENT_FATAL_NO_CMD_ACK:
        {
            wifi_driver_error("\n\r[ERROR] - FATAL ERROR: No Cmd Ack detected "
                        "[cmd opcode = 0x%x] \n\r",
                                        slFatalErrorEvent->Data.NoCmdAck.Code);
        }
        break;

        case SL_DEVICE_EVENT_FATAL_SYNC_LOSS:
        {
            wifi_driver_error("\n\r[ERROR] - FATAL ERROR: Sync loss detected n\r");
        }
        break;

        case SL_DEVICE_EVENT_FATAL_CMD_TIMEOUT:
        {
            wifi_driver_error("\n\r[ERROR] - FATAL ERROR: Async event timeout detected "
                        "[event opcode =0x%x]  \n\r",
                                    slFatalErrorEvent->Data.CmdTimeout.Code);
        }
        break;

        default:
            wifi_driver_error("\n\r[ERROR] - FATAL ERROR: Unspecified error detected \n\r");
        break;
    }

#ifdef _DEBUG_
    while(1);
#endif
}

/*!
    \brief          SimpleLinkNetAppRequestEventHandler

    This handler gets called whenever a NetApp event is reported
    by the NWP / Host driver. User can write he's logic to handle
    the event here.

    \param          pNetAppRequest     -   Pointer to NetApp request structure.

    \param          pNetAppResponse    -   Pointer to NetApp request Response.

    \note           For more information, please refer to: user.h in the porting
                    folder of the host driver and the  CC3120/CC3220 NWP programmer's
                    guide (SWRU455) section 17.9.

    \return         void

*/
void SimpleLinkNetAppRequestEventHandler(SlNetAppRequest_t *pNetAppRequest, SlNetAppResponse_t *pNetAppResponse)
{
    /* Unused in this application */
}

/*!
    \brief          SimpleLinkNetAppRequestMemFreeEventHandler

    This handler gets called whenever the NWP is done handling with
    the buffer used in a NetApp request. This allows the use of
    dynamic memory with these requests.

    \param          pNetAppRequest     -   Pointer to NetApp request structure.

    \param          pNetAppResponse    -   Pointer to NetApp request Response.

    \note           For more information, please refer to: user.h in the porting
                    folder of the host driver and the  CC3120/CC3220 NWP programmer's
                    guide (SWRU455) section 17.9.

    \return         void

*/
void SimpleLinkNetAppRequestMemFreeEventHandler(uint8_t *buffer)
{
    /* Unused in this application */
	wifi_driver_debug("\n\r[MEM FREE EVENT]  ! \r\n");
}

void SimpleLinkSocketTriggerEventHandler(SlSockTriggerEvent_t* ev)
{
	/* Manually created by vv */
	wifi_driver_debug("\n\r[SOCKET TRIGGER EVENT]  Ev = %u\r\n", ev->Event);
}
